@@ -122,11 +122,23 @@ def write_dataframe_to_table(
         Exception: If the write operation fails
     """
     try:
+        # DEBUG: Log input DataFrame information
+        logger.debug(f"🔍 DEBUG: Input DataFrame - Type: {type(df).__name__}, Is DataFrame: {isinstance(df, pd.DataFrame)}")
+        if isinstance(df, pd.DataFrame):
+            logger.debug(f"   Shape: {df.shape}")
+        else:
+            logger.debug(f"   Value: {df}")
+
+        sd = df.head(1).to_dict()
+        logger.error(f"   here is the sample data: {str(sd)[:500]}")
+
+
         # Format DataFrame using field_types.format_sync_file if df_fields provided
         if df_fields is not None:
             try:
                 from lht.util import field_types
                 df = field_types.format_sync_file(df, df_fields)
+                logger.debug(f"🔍 DEBUG: After format_sync_file - Type: {type(df).__name__}, Shape: {df.shape if isinstance(df, pd.DataFrame) else 'N/A'}")
                 
                 # VALIDATION: Ensure snowflake_fields is provided when df_fields is available
                 if snowflake_fields is None or len(snowflake_fields) == 0:
@@ -143,11 +155,11 @@ def write_dataframe_to_table(
         if validate_types:
             validation_result = validate_dataframe_types(df)
             if not validation_result['is_valid']:
-                logger.warning("⚠️ DataFrame type validation found potential issues:")
-                for issue in validation_result['issues']:
-                    logger.warning(f"   - {issue}")
-                for recommendation in validation_result['recommendations']:
-                    logger.info(f"   💡 {recommendation}")
+            #     logger.warning("⚠️ DataFrame type validation found potential issues:")
+            #     for issue in validation_result['issues']:
+            #         logger.warning(f"   - {issue}")
+            #     for recommendation in validation_result['recommendations']:
+            #         logger.info(f"   💡 {recommendation}")
                 
                 # If there are critical issues, consider using more lenient settings
                 if use_logical_type and any("Mixed data types" in issue for issue in validation_result['issues']):
@@ -192,7 +204,6 @@ def write_dataframe_to_table(
                 
 
                 # Use centralized table creation
-                print(f"🔍 About to call table_creator.ensure_table_exists_for_dataframe with force_full_sync={force_full_sync}")
                 table_creator.ensure_table_exists_for_dataframe(
                     session=session,
                     schema=schema,
@@ -202,7 +213,7 @@ def write_dataframe_to_table(
                     force_full_sync=force_full_sync,  # Pass through the force_full_sync parameter
                     database=current_db
                 )
-                print(f"✅ Table creation completed, now writing data with overwrite={overwrite}")
+                logger.info(f"✅ Table creation completed, now writing data with overwrite={overwrite}")
                 
                 # Now write to the existing table using the already-processed DataFrame
                 # Convert all datetime fields to timezone-naive for Snowflake compatibility
@@ -215,12 +226,15 @@ def write_dataframe_to_table(
                                 logger.error(f"❌ Failed to convert {col} to timezone-naive: {e}")
                 
                 #df['CREATEDDATE'] = pd.to_datetime(df['CREATEDDATE'], errors='coerce')
+                # DEBUG: Log df before creating Snowpark DataFrame
+                logger.debug(f"🔍 DEBUG: Before create_dataframe (primary) - Type: {type(df).__name__}, Shape: {df.shape if isinstance(df, pd.DataFrame) else 'N/A'}")
+                
                 # Convert to Snowpark DataFrame and use save_as_table instead of write_pandas
-                print(f"🔍 Writing data with mode: {'overwrite' if overwrite else 'append'}")
-                print(f"🔍 Target table: {full_table_name}")
                 snowpark_df = session.create_dataframe(df)
+                df = None
                 snowpark_df.write.mode("overwrite" if overwrite else "append").save_as_table(full_table_name)
-                print(f"✅ Data written successfully with mode: {'overwrite' if overwrite else 'append'}")
+                snowpark_df = None
+                logger.info(f"✅ Data written successfully with mode: {'overwrite' if overwrite else 'append'}")
                 result = True  # save_as_table doesn't return a result object like write_pandas
             except Exception as table_error:
                 logger.warning(f"Centralized table creation failed, falling back to auto-create: {table_error}")
@@ -247,12 +261,15 @@ def write_dataframe_to_table(
                 #             except Exception as e:
                 #                 logger.error(f"❌ Failed to convert {col} to timezone-naive: {e}")
                 
+                # DEBUG: Log df_fallback before creating Snowpark DataFrame
+                logger.debug(f"🔍 DEBUG: Before create_dataframe (fallback) - Type: {type(df_fallback).__name__}, Shape: {df_fallback.shape if isinstance(df_fallback, pd.DataFrame) else 'N/A'}")
+                
                 # Convert to Snowpark DataFrame and use save_as_table instead of write_pandas
-                print(f"🔍 Writing fallback data with mode: {'overwrite' if overwrite else 'append'}")
-                print(f"🔍 Target table: {full_table_name}")
                 snowpark_df = session.create_dataframe(df_fallback)
+                df_fallback = None
                 snowpark_df.write.mode("overwrite" if overwrite else "append").save_as_table(full_table_name)
-                print(f"✅ Fallback data written successfully with mode: {'overwrite' if overwrite else 'append'}")
+                snowpark_df = None
+                logger.info(f"✅ Fallback data written successfully with mode: {'overwrite' if overwrite else 'append'}")
                 result = True  # save_as_table doesn't return a result object like write_pandas
         else:
             # Use original behavior, but ensure DataFrame is properly processed
@@ -275,12 +292,81 @@ def write_dataframe_to_table(
                         except Exception as e:
                             logger.error(f"❌ Failed to convert {col} to timezone-naive: {e}")
             
+            # DEBUG: Log df_processed before creating Snowpark DataFrame
+            logger.debug(f"🔍 DEBUG: Before create_dataframe (processed) - Type: {type(df_processed).__name__}, Shape: {df_processed.shape if isinstance(df_processed, pd.DataFrame) else 'N/A'}")
+            logger.debug(f"🔍 DEBUG: df_processed columns count: {len(df_processed.columns)}")
+            logger.debug(f"🔍 DEBUG: df_processed type check - isinstance DataFrame: {isinstance(df_processed, pd.DataFrame)}")
+            
+            # Ensure DataFrame is properly reset and clean for Snowflake
+            df_processed = df_processed.copy()
+            df_processed.reset_index(drop=True, inplace=True)
+            logger.debug(f"🔍 DEBUG: df_processed shape: {df_processed.shape}")
+            logger.debug(f"🔍 DEBUG: df_processed empty check: {df_processed.empty}")
+            
+            # Check for problematic dtypes
+            logger.debug(f"🔍 DEBUG: Checking for problematic dtypes...")
+            for col in df_processed.columns:
+                dtype = df_processed[col].dtype
+                if str(dtype) == 'object':
+                    # Check if any values are not basic types
+                    sample = df_processed[col].dropna().head(3)
+                    for val in sample:
+                        if val is not None and not pd.isna(val):
+                            val_type = type(val).__name__
+                            if val_type not in ['str', 'int', 'float', 'bool', 'Timestamp', 'datetime', 'date']:
+                                logger.debug(f"🔍 DEBUG: Found unusual type in {col}: {val_type} - {str(val)[:100]}")
+
             # Convert to Snowpark DataFrame and use save_as_table instead of write_pandas
-            print(f"🔍 Writing processed data with mode: {'overwrite' if overwrite else 'append'}")
-            print(f"🔍 Target table: {full_table_name}")
-            snowpark_df = session.create_dataframe(df_processed)
+            try:
+                # Final validation - ensure it's a proper pandas DataFrame
+                if not isinstance(df_processed, pd.DataFrame):
+                    raise TypeError(f"df_processed is not a pandas DataFrame, got {type(df_processed)}")
+                
+                # Ensure DataFrame is in a clean state
+                if df_processed.index.name is not None:
+                    df_processed.reset_index(drop=True, inplace=True)
+                
+                logger.debug(f"🔍 DEBUG: Attempting create_dataframe with type: {type(df_processed)}")
+                logger.debug(f"🔍 DEBUG: DataFrame has index: {df_processed.index.tolist()[:5]}")
+                
+                # DEBUG: Try write_pandas first to a temporary table to validate DataFrame
+                try:
+                    debug_temp_table = f"{temp_table}_DEBUG" if temp_table else f"{table}_DEBUG"
+                    logger.debug(f"🔍 DEBUG: Testing DataFrame with write_pandas to {schema}.{debug_temp_table}")
+                    
+                    session.write_pandas(
+                        df_processed, 
+                        table_name=debug_temp_table,
+                        schema=schema,
+                        auto_create_table=True,
+                        overwrite=True
+                    )
+                    logger.debug(f"✅ DEBUG: write_pandas succeeded - DataFrame is valid")
+                    logger.debug(f"🔍 DEBUG: Table {schema}.{debug_temp_table} created for inspection - NOT dropping")
+                    # Table will remain for manual inspection - you can drop it manually after reviewing
+                except Exception as wp_error:
+                    logger.error(f"❌ DEBUG: write_pandas also failed: {wp_error}")
+                    logger.error(f"   This suggests the DataFrame itself has structural issues")
+                
+                snowpark_df = session.create_dataframe(df_processed)
+            except Exception as create_error:
+                logger.error(f"❌ create_dataframe failed: {create_error}")
+                logger.error(f"   DataFrame type: {type(df_processed)}")
+                logger.error(f"   DataFrame shape: {df_processed.shape}")
+                logger.error(f"   DataFrame columns: {list(df_processed.columns)[:10]}...")
+                # Try to identify the problematic data
+                try:
+                    import json
+                    # Try to convert to dict to see what's actually in there
+                    sample_dict = df_processed.head(1).to_dict()
+                    logger.error(f"   First row sample: {str(sample_dict)[:500]}")
+                except Exception as dict_error:
+                    logger.error(f"   Could not convert to dict: {dict_error}")
+                raise
+            df_processed = None
             snowpark_df.write.mode("overwrite" if overwrite else "append").save_as_table(full_table_name)
-            print(f"✅ Processed data written successfully with mode: {'overwrite' if overwrite else 'append'}")
+            snowpark_df = None
+            logger.info(f"✅ Processed data written successfully with mode: {'overwrite' if overwrite else 'append'}")
             result = True  # save_as_table doesn't return a result object like write_pandas
         
         logger.debug(f"✅ Successfully wrote {len(df)} records to {full_table_name}")
@@ -441,11 +527,13 @@ def write_dataframe_with_type_handling(
 def write_batch_to_temp_table(
     session: Session,
     df: pd.DataFrame,
-    schema: str,
+    schema: Optional[str],
     temp_table: str,
     df_fields: dict,
     validate_types: bool = True,
-    force_full_sync: bool = False
+    force_full_sync: bool = False,
+    main_table: str = None,
+    snowflake_fields: Optional[dict] = None
 ) -> bool:
     """
     Write a batch DataFrame to a temporary table for incremental sync operations.
@@ -453,29 +541,52 @@ def write_batch_to_temp_table(
     Args:
         session: Snowflake Snowpark session
         df: DataFrame to write
-        schema: Target schema name
+        schema: Target schema name (None to use session's current schema)
         temp_table: Temporary table name
         df_fields: List of field names for validation
         validate_types: Whether to validate DataFrame types before writing
+        force_full_sync: Whether this is a full sync operation
+        main_table: Main table name to copy schema from (for temp table creation)
+        snowflake_fields: Salesforce field type definitions
         
     Returns:
         bool: True if successful
     """
-    logger.debug(f"📦 Writing batch to temp table: {schema}.{temp_table}")
+    # Get current schema if not provided
+    if schema is None:
+        current_schema = session.sql('SELECT CURRENT_SCHEMA()').collect()[0][0]
+        logger.debug(f"📦 Using current schema: {current_schema}")
+    else:
+        current_schema = schema
+        
+    logger.debug(f"📦 Writing batch to temp table: {current_schema}.{temp_table}")
+    
+    # Create temporary table with schema copied from main table if provided
+    if main_table:
+        try:
+            create_temp_query = f"CREATE OR REPLACE TEMPORARY TABLE {temp_table} LIKE {main_table}"
+            logger.debug(f"🔍 Creating temp table with schema copy: {create_temp_query}")
+            session.sql(create_temp_query).collect()
+            logger.debug(f"✅ Temp table created with schema from {main_table}")
+        except Exception as e:
+            logger.warning(f"⚠️ Failed to create temp table with schema copy: {e}")
+            logger.warning(f"⚠️ Falling back to auto-create mode")
+            main_table = None  # Fall back to auto-create
     
     try:
         return write_dataframe_to_table(
             session=session,
             df=df,
-            schema=schema,
+            schema=current_schema,
             table=temp_table,
             overwrite=False,
-            auto_create=False,
+            auto_create=main_table is None,  # Only auto-create if we didn't create with LIKE
             temp_table=temp_table,
             validate_types=validate_types,
             use_logical_type=False,  # More lenient for temp tables
             on_error="CONTINUE",
             df_fields=df_fields,  # Pass field definitions for proper formatting
+            snowflake_fields=snowflake_fields,  # Pass Salesforce field types
             force_full_sync=force_full_sync  # Pass through force_full_sync parameter
         )
     except Exception as e:
@@ -489,7 +600,7 @@ def write_batch_to_temp_table(
             return write_dataframe_to_table(
                 session=session,
                 df=df_standardized,
-                schema=schema,
+                schema=current_schema,
                 table=temp_table,
                 overwrite=False,
                 auto_create=False,
@@ -498,6 +609,7 @@ def write_batch_to_temp_table(
                 use_logical_type=False,
                 on_error="CONTINUE",
                 df_fields=df_fields,  # Pass field definitions for proper formatting
+                snowflake_fields=snowflake_fields,  # Pass Salesforce field types
                 force_full_sync=force_full_sync  # Pass through force_full_sync parameter
             )
         else:
@@ -535,17 +647,17 @@ def write_batch_to_main_table(
     if force_full_sync and is_first_batch:
         # Force full sync: drop and recreate table, then overwrite first batch
         overwrite = True
-        print(f"💾 Force full sync: overwriting first batch to recreate table")
+        logger.debug(f"💾 Force full sync: overwriting first batch to recreate table")
     elif is_first_batch:
         # Regular first batch: append to existing or new table
         overwrite = False
-        print(f"💾 Regular first batch: appending to table")
+        logger.debug(f"💾 Regular first batch: appending to table")
     else:
         # Subsequent batches: always append
         overwrite = False
-        print(f"💾 Subsequent batch: appending to table")
+        logger.debug(f"💾 Subsequent batch: appending to table")
     
-    print(f"💾 Writing batch to main table: {schema}.{table} (overwrite={overwrite}, is_first_batch={is_first_batch}, force_full_sync={force_full_sync})")
+    logger.debug(f"💾 Writing batch to main table: {schema}.{table} (overwrite={overwrite}, is_first_batch={is_first_batch}, force_full_sync={force_full_sync})")
     
     return write_dataframe_to_table(
         session=session,
